#ifndef _CONFIG_H
#define _CONFIG_H

#ifdef __linux__
#define _GNU_SOURCE
#endif

#include <stdio.h>
#include <fcntl.h>
#include <stdlib.h>
#include <stdarg.h>
#include <stddef.h>		/* offsetof() */
#include <getopt.h>
#include <unistd.h>
#include <string.h>
#include <sys/select.h>
#include <math.h>
#ifdef HAVE_IFADDRS
#include <ifaddrs.h>	/* local IP */
#endif
#ifdef __FreeBSD__
#include <sys/sockio.h>	/* SIOCGIFCONF on FreeBSD */
#endif
#include <time.h>
#include <signal.h>
#include <errno.h>		/* EINPROGRESS */

#include <netdb.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <sys/uio.h>
#include <netinet/in.h>
#include <netinet/in_systm.h>
#include <netinet/ip.h>
#include <netinet/ip_icmp.h>
#include <net/if.h>
#include <netinet/udp.h>
#include <arpa/inet.h>

#define SMAP_VERSION "0.4.1"

/* error() helper stuff */
enum { ERR_DEBUG, ERR_NOTICE, ERR_FUNC, ERR_ERROR, ERR_FATAL };
#define ERRBUF  1024
#define DEBUG (config.debug > 0)
#define FUNCTION(x,y) { if (config.debug > 1) { fprintf(stderr, "%s:%s() invoked\n", x,y );}}

/* scan mode */
enum { MODE_NONE = 0, MODE_IP, MODE_HOST, MODE_NET };

struct config_t {
	unsigned int debug;

#define FLAG_EMPTY				0x0000
#define FLAG_NOICMPPING			0x0001
#define FLAG_FINGERPRINT		0x0002
#define FLAG_FINGERPRINT_SMALL	0x0004
#define FP_ENABLED	(config.flags & FLAG_FINGERPRINT)
#define FLAG_FPLEARNING			0x0008
#define IS_LEARNING (config.flags & FLAG_FPLEARNING)
#define FLAG_NOROOT				0x0010	/* don't have root, so don't try
										 * ICMP ping or rawsocket */
#define FLAG_SIPPORTBUSY		0x0020	/* port 5060 in use
										 * flag set, if error reported */
#define FLAG_UDP_TRANSPORT		0x0040
#define FLAG_TCP_TRANSPORT		0x0080
#define FLAG_TLS_TRANSPORT		0x0100	/* not used yet */

	int flags;

	/* targets */
	unsigned int mode;
	char *target_host;
	struct in_addr net_ip;
	struct in_addr net_base;
	unsigned int net_mask;

	/* sip */
#define DEFAULT_PORT	5060
	unsigned int sip_port;
	char *sip_domain;
#define DEFAULT_TIMEOUT	500
	unsigned int timeout;	/* in msec */

	char *localip;			/* local IP as string */

	/* request */
	char *requestfile;
} config;

struct stats_t {
	unsigned int scanned;
	unsigned int up;
	unsigned int sip;
} stats;

#define SIPLEN		4096	/* based on RFC */
#define SIPMINLEN	64		/* minimum size to fit a SIP message in */
enum { SIP_OPTIONS = 1, SIP_BROKENFROMTO, SIP_NEWMETHOD, SIP_PRACK, \
		SIP_INVITE, SIP_PING, SIP_TMP };

/* payload len is based on what FreeBSD sends by default */
#define ICMP_PAYLOAD_LEN	60

/*** transport protocols *****************************************************/

enum { TRANSPORT_UDP = 1, TRANSPORT_TCP };
#define TRANSPORT_STRING(x) ((x == TRANSPORT_UDP) ? "UDP" : "TCP" )

/*** randomizer stuff ********************************************************/
enum { RAND_CALLID = 1, RAND_TAG, RAND_CSEQ };
#define CALLID_LEN	64
#define TAG_LEN		64

/*** fingerprint *************************************************************/
struct fingerprint_t {
	char			*name;	/* name of the UA identified */

	int				options;
	int				newmethod;
	int				brokenfromto;
	int				prack;
	int				invite;
	int				ping;
	int				tmp;			/* temporary scratchpad */

	int				allow_class;
	int				supported_class;
	int				hoe_class;		/* header order/existance */

	char			*header_ua;		/* pointers to useful headers */
	char			*header_server;

	struct fingerprint_t	*next;	/* next fingerprint */
};
typedef struct fingerprint_t fingerprint;

/* we have four fingerprint identicators */
#define FP_PERFECTMATCH	4
#define NO_RESPONSE -1
#define NO_RELEVANCE -2

#include "prototypes.h"

#endif /* _CONFIG_H */
